@@ -14,10 +14,7 @@
 
 use tokio::net::{TcpStream, ToSocketAddrs};
 
-use crate::{
-    command::{Del, FlushAll, Get, MGet, MSet, MSetNx, Ping, SetOption},
-    Connection, Error, Model, Result,
-};
+use crate::{command::*, Connection, Error, Model, Result};
 
 pub struct Client {
     connection: Connection,
@@ -38,17 +35,25 @@ impl Client {
         }
     }
 
-    pub async fn get(&mut self, key: Vec<u8>) -> Result<Option<Vec<u8>>> {
-        self.connection.send(Get::new(key)).await?;
+    pub async fn get<In, Out>(&mut self, key: In) -> Result<Option<Out>>
+    where
+        In: Into<Vec<u8>>,
+        Out: From<Vec<u8>>,
+    {
+        self.connection.send(Get::new(key.into())).await?;
         match self.connection.recv().await? {
-            Some(Model::String(result)) => Ok(Some(result)),
+            Some(Model::String(result)) => Ok(Some(result.into())),
             Some(Model::Nil) => Ok(None),
             model => match_failure(model),
         }
     }
 
-    pub async fn set(&mut self, key: Vec<u8>, value: Vec<u8>, option: SetOption) -> Result<bool> {
-        let set = option.build(key, value, false);
+    pub async fn set<In0, In1>(&mut self, key: In0, value: In1, option: SetOption) -> Result<bool>
+    where
+        In0: Into<Vec<u8>>,
+        In1: Into<Vec<u8>>,
+    {
+        let set = option.build(key.into(), value.into(), false);
         self.connection.send(set).await?;
         match self.connection.recv().await? {
             Some(Model::Status(status)) if status.eq_ignore_ascii_case("OK") => Ok(true),
@@ -57,17 +62,22 @@ impl Client {
         }
     }
 
-    pub async fn get_set(
+    pub async fn get_set<In0, In1, Out>(
         &mut self,
-        key: Vec<u8>,
-        value: Vec<u8>,
+        key: In0,
+        value: In1,
         option: SetOption,
-    ) -> Result<Option<Vec<u8>>> {
-        let set = option.build(key, value, true);
+    ) -> Result<Option<Out>>
+    where
+        In0: Into<Vec<u8>>,
+        In1: Into<Vec<u8>>,
+        Out: From<Vec<u8>>,
+    {
+        let set = option.build(key.into(), value.into(), true);
         self.connection.send(set).await?;
         match self.connection.recv().await? {
             Some(Model::Nil) => Ok(None),
-            Some(Model::String(result)) => Ok(Some(result)),
+            Some(Model::String(result)) => Ok(Some(result.into())),
             model => match_failure(model),
         }
     }
@@ -80,15 +90,39 @@ impl Client {
         }
     }
 
-    pub async fn del(&mut self, keys: Vec<Vec<u8>>) -> Result<i64> {
+    pub async fn del<In>(&mut self, keys: Vec<In>) -> Result<u64>
+    where
+        In: Into<Vec<u8>>,
+    {
+        let keys = keys.into_iter().map(|k| k.into()).collect();
         self.connection.send(Del::new(keys)).await?;
         match self.connection.recv().await? {
-            Some(Model::Integer(result)) => Ok(result),
+            Some(Model::Integer(result)) if result >= 0 => Ok(result as u64),
             model => match_failure(model),
         }
     }
 
-    pub async fn mset(&mut self, kvs: Vec<(Vec<u8>, Vec<u8>)>) -> Result<()> {
+    pub async fn exists<In>(&mut self, keys: Vec<In>) -> Result<u64>
+    where
+        In: Into<Vec<u8>>,
+    {
+        let keys = keys.into_iter().map(|k| k.into()).collect();
+        self.connection.send(Exists::new(keys)).await?;
+        match self.connection.recv().await? {
+            Some(Model::Integer(result)) if result >= 0 => Ok(result as u64),
+            model => match_failure(model),
+        }
+    }
+
+    pub async fn mset<In0, In1>(&mut self, kvs: Vec<(In0, In1)>) -> Result<()>
+    where
+        In0: Into<Vec<u8>>,
+        In1: Into<Vec<u8>>,
+    {
+        let kvs = kvs
+            .into_iter()
+            .map(|kv| (kv.0.into(), kv.1.into()))
+            .collect();
         self.connection.send(MSet::new(kvs)).await?;
         match self.connection.recv().await? {
             Some(Model::Status(status)) if status.eq_ignore_ascii_case("OK") => Ok(()),
@@ -96,7 +130,15 @@ impl Client {
         }
     }
 
-    pub async fn msetnx(&mut self, kvs: Vec<(Vec<u8>, Vec<u8>)>) -> Result<bool> {
+    pub async fn msetnx<In0, In1>(&mut self, kvs: Vec<(In0, In1)>) -> Result<bool>
+    where
+        In0: Into<Vec<u8>>,
+        In1: Into<Vec<u8>>,
+    {
+        let kvs = kvs
+            .into_iter()
+            .map(|kv| (kv.0.into(), kv.1.into()))
+            .collect();
         self.connection.send(MSetNx::new(kvs)).await?;
         match self.connection.recv().await? {
             Some(Model::Integer(result)) => match result {
@@ -108,7 +150,12 @@ impl Client {
         }
     }
 
-    pub async fn mget(&mut self, keys: Vec<Vec<u8>>) -> Result<Vec<Option<Vec<u8>>>> {
+    pub async fn mget<In, Out>(&mut self, keys: Vec<In>) -> Result<Vec<Option<Out>>>
+    where
+        In: Into<Vec<u8>>,
+        Out: From<Vec<u8>>,
+    {
+        let keys = keys.into_iter().map(|k| k.into()).collect();
         self.connection.send(MGet::new(keys)).await?;
         match self.connection.recv().await? {
             Some(Model::Array(models)) => {
@@ -116,7 +163,7 @@ impl Client {
                 for model in models.into_iter() {
                     match model {
                         Model::Nil => result.push(None),
-                        Model::String(v) => result.push(Some(v)),
+                        Model::String(v) => result.push(Some(v.into())),
                         _ => return match_failure(Some(model)),
                     }
                 }
@@ -125,6 +172,64 @@ impl Client {
             model => match_failure(model),
         }
     }
+
+    pub async fn strlen<In>(&mut self, key: In) -> Result<u64>
+    where
+        In: Into<Vec<u8>>,
+    {
+        self.connection.send(Strlen::new(key.into())).await?;
+        match self.connection.recv().await? {
+            Some(Model::Integer(len)) if len >= 0 => Ok(len as u64),
+            model => match_failure(model),
+        }
+    }
+
+    pub async fn get_range<In, Out>(&mut self, key: In, start: i64, end: i64) -> Result<Out>
+    where
+        In: Into<Vec<u8>>,
+        Out: From<Vec<u8>>,
+    {
+        self.connection
+            .send(GetRange::new(key.into(), start, end))
+            .await?;
+        match self.connection.recv().await? {
+            Some(Model::String(result)) => Ok(result.into()),
+            model => match_failure(model),
+        }
+    }
+
+    pub async fn set_range<In0, In1>(
+        &mut self,
+        key: In0,
+        index: i64,
+        substitute: In1,
+    ) -> Result<u64>
+    where
+        In0: Into<Vec<u8>>,
+        In1: Into<Vec<u8>>,
+    {
+        self.connection
+            .send(SetRange::new(key.into(), index, substitute.into()))
+            .await?;
+        match self.connection.recv().await? {
+            Some(Model::Integer(len)) if len >= 0 => Ok(len as u64),
+            model => match_failure(model),
+        }
+    }
+
+    pub async fn append<In0, In1>(&mut self, key: In0, suffix: In1) -> Result<u64>
+    where
+        In0: Into<Vec<u8>>,
+        In1: Into<Vec<u8>>,
+    {
+        self.connection
+            .send(Append::new(key.into(), suffix.into()))
+            .await?;
+        match self.connection.recv().await? {
+            Some(Model::Integer(len)) if len >= 0 => Ok(len as u64),
+            model => match_failure(model),
+        }
+    }
 }
 
 fn match_failure<T>(model: Option<Model>) -> Result<T> {
