@@ -12,24 +12,34 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+mod append;
 mod del;
+mod exists;
 mod flushall;
 mod get;
+mod getrange;
 mod mget;
 mod mset;
 mod msetnx;
 mod ping;
 mod set;
+mod setrange;
+mod strlen;
 
+pub use append::Append;
 use bytes::{BufMut, BytesMut};
 pub use del::Del;
+pub use exists::Exists;
 pub use flushall::FlushAll;
 pub use get::Get;
+pub use getrange::GetRange;
 pub use mget::MGet;
 pub use mset::MSet;
 pub use msetnx::MSetNx;
 pub use ping::Ping;
 pub use set::{Set, SetOption};
+pub use setrange::SetRange;
+pub use strlen::Strlen;
 
 pub trait Command {
     fn as_bytes(&self) -> Vec<u8>;
